==> javautils-gui-1.0.4-sources.jar: tecgraf/javautils/gui/table/ColumnsObjectTableModel.java <==
package tecgraf.javautils.gui.table;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import javax.swing.event.TableModelEvent;
import javax.swing.table.AbstractTableModel;

/**
 * <p>
 * Modelo de tabela de objetos de fcil construo.<br>
 * Sua arquitetura permite reutilizar o cdigo das colunas e exime o
 * desenvolvedor de trabalhar com os ndices da tabela, seja de linha ou de
 * coluna.
 * </p>
 * <p>
 * Para criar um modelo de tabela voc precisa de duas coisas:
 * <ul>
 * <li>os modelos das colunas que podem ser criados implementando
 * {@link IColumn} ou estendendo {@link AbstractColumn}.</li>
 * <li>uma lista com os objetos que devero aparecer na tabela.</li>
 * </ul>
 * </p>
 * <p>
 * Obs. esse modelo no faz uso dos mtodos
 * {@link IColumn#createTableCellEditor()} e
 * {@link IColumn#createTableCellRenderer()}.
 * </p>
 * <p>
 * Antes de utilizar esse modelo d uma olhada na classe
 * {@link ObjectTableBuilder} que facilita a construo de tabelas cujas
 * informaes mostradas esto contidas em um objeto.
 * </p>
 * 
 * @param <T> Tipo dos objetos que sero representados por linhas da tabela.
 * 
 * @see ColumnsObjectTableModelSample 
 * 
 * @author Tecgraf
 */
public class ColumnsObjectTableModel<T> extends AbstractTableModel {
  /** Lista de objetos representando linhas da tabela */
  private List<T> rows;
  /** Modelo das colunas da tabela na ordem em que devem aparecer. */
  private IColumn<T>[] columns;

  /**
   * Cria o modelo.
   * 
   * @param rows lista de objetos-linha.
   * @param columns Modelo das colunas da tabela na ordem em que devem aparecer.
   * 
   * @throws IllegalArgumentException se a lista de linhas ou o provedor forem
   *         iguais a null, ou se a lista de nomes das colunas retornada pelo
   *         provedor for null
   */
  public ColumnsObjectTableModel(List<T> rows, IColumn<T>... columns) {
    if (rows == null) {
      throw new IllegalArgumentException("rows == null");
    }
    if (null == columns) {
      throw new IllegalArgumentException("columns == null");
    }
    if (columns.length == 0) {
      throw new IllegalArgumentException("columns.length == 0");
    }

    this.rows = rows;
    this.columns = columns;
  }

  /**
   * Obtm os modelos das colunas da tabela na ordem em que elas aparecem.<br>
   * Obs. mudanas nesse array e/ou nas colunas poderam danificar o
   * funcionamento da tabela.
   * 
   * @return os modelos das colunas da tabela na ordem em que elas aparecem.
   */
  public IColumn<T>[] getColumns() {
    return columns;
  }

  /* A partir daqui comea a implementao dos mtodos de AbstractTableModel. */

  /**
   * {@inheritDoc}
   */
  @Override
  public int getRowCount() {
    return rows.size();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public int getColumnCount() {
    return columns.length;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Object getValueAt(int rowIndex, int columnIndex) {
    T row = rows.get(rowIndex);
    if (null == row) {
      throw new NullPointerException("rowIndex points to a null object.");
    }

    return columns[columnIndex].getValue(row);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void setValueAt(Object newValue, int rowIndex, int columnIndex) {
    T row = rows.get(rowIndex);
    if (null == row) {
      throw new NullPointerException("rowIndex points to a null object.");
    }

    columns[columnIndex].setValue(row, newValue);
    fireTableCellUpdated(rowIndex, columnIndex);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean isCellEditable(int rowIndex, int columnIndex) {
    T row = rows.get(rowIndex);
    if (null == row) {
      throw new NullPointerException("rowIndex points to a null object.");
    }

    return columns[columnIndex].isEditable(row);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String getColumnName(int columnIndex) {
    return columns[columnIndex].getColumnName();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Class<?> getColumnClass(int columnIndex) {
    return columns[columnIndex].getColumnClass();
  }

  /* Aqui termina a implementao dos mtodos de AbstractTableModel. */

  /* A partir daqui comeam os mtodos para alterar a lista de objetos-linha. */

  /**
   * Remove todos os elementos de objetos.
   */
  public void clear() {
    int size = rows.size();
    this.rows = new ArrayList<T>();
    if (size > 0) {
      fireTableRowsDeleted(0, size - 1);
    }
  }

  /**
   * Substitui a lista de objetos-linha.
   * 
   * @param newRows nova lista de objetos-linha.
   * 
   * @throws IllegalArgumentException se a lista de novas linhas for null
   */
  public void setRows(List<T> newRows) {
    if (newRows == null) {
      throw new IllegalArgumentException("newRows == null");
    }
    this.rows = newRows;
    fireTableChanged(new TableModelEvent(this));
  }

  /**
   * Adiciona um objeto-linha  lista. Se j existir um objeto com o mesmo
   * identificador, este  sobrescrito (no permite duplicatas).
   * 
   * @param row objeto-linha a ser adicionado.
   * 
   * @throws IllegalArgumentException se <code>row</code> for nulo.
   */
  public void add(T row) {
    add(row, true);
  }

  /**
   * Adiciona uma nova linha  lista, com a possibilidade de notificar ou no a
   * tabela.
   * 
   * @param row nova linha
   * @param notify <code>true</code> se a tabela deve ser notificada da insero
   */
  private void add(T row, boolean notify) {
    if (row == null) {
      throw new IllegalArgumentException("row == null");
    }
    int index = rows.indexOf(row);
    if (index != -1) {
      rows.set(index, row);
      if (notify) {
        fireTableRowsUpdated(index, index);
      }
    }
    else {
      rows.add(row);
      if (notify) {
        index = rows.size() - 1;
        fireTableRowsInserted(index, index);
      }
    }
  }

  /**
   * Adiciona uma coleo de objetos-linha  lista. Se j existir um objeto com
   * o mesmo identificador, este  sobrescrito (no permite duplicatas).
   * 
   * @param rows coleo de objetos-linha a serem adicionados.
   * 
   * @throws IllegalArgumentException se <code>rows</code> for nulo.
   */
  public void addAll(Collection<T> rows) {
    if (rows == null) {
      throw new IllegalArgumentException("rows == null");
    }
    for (T row : rows) {
      /*
       * inserimos a linha sem notificar a tabela, pois faremos uma nica
       * notificao aps o loop
       */
      add(row, false);
    }
    fireTableChanged(new TableModelEvent(this));
  }

  /**
   * Remove um objeto-linha da lista.
   * 
   * @param row objeto-linha a ser removido.
   * 
   * @return true se o objeto-linha for encontrado e removido com sucesso, false
   *         caso contrrio.
   * 
   * @throws IllegalArgumentException se <code>row</code> for nulo.
   */
  public boolean remove(T row) {
    if (row == null) {
      throw new IllegalArgumentException("row == null");
    }
    int index = rows.indexOf(row);
    if (index == -1) {
      return false;
    }
    rows.remove(index);
    fireTableRowsDeleted(index, index);
    return true;
  }

  /**
   * Remove um objeto-linha da lista pelo seu ndice do modelo.
   * 
   * @param rowIndex ndice do objeto-linha a ser removido.
   * 
   * @return o objeto-linha removido.
   * 
   * @throws IndexOutOfBoundsException se o ndice for negativo ou se for maior
   *         do que o tamanho da lista de objeto-linha.
   */
  public T remove(int rowIndex) {
    T row = rows.remove(rowIndex);
    fireTableRowsDeleted(rowIndex, rowIndex);
    return row;
  }

  /**
   * Remove os objetos-linha especificados pelos seus ndices.
   * 
   * @see AbstractTableModel#fireTableRowsDeleted(int, int)
   * @param rowIndexes ndices dos objetos-linha a serem removidos.
   * 
   * @return coleo de objetos removidos.
   * 
   * @throws IllegalArgumentException se <code>rowIndexes</code> for nulo.
   * @throws IndexOutOfBoundsException se algum ndice for negativo ou se for
   *         maior do que o tamanho da lista de objeto-linha.
   * 
   */
  public Collection<T> removeAll(int[] rowIndexes) {
    if (rowIndexes == null) {
      throw new IllegalArgumentException("rowIndexes == null");
    }
    List<T> removedRows = new ArrayList<T>();
    if (rowIndexes.length > 0) {
      /*
       * devemos sinalizar mudanas apenas no intervalo que compreende as linhas
       * removidas (inclusive). Para isso, precisamos achar o menor e o maior
       * ndices dentre os reportados.
       */
      int start = Integer.MAX_VALUE, end = 0;
      for (int rowIndex : rowIndexes) {
        removedRows.add(rows.get(rowIndex));
        start = Math.min(start, rowIndex);
        end = Math.max(end, rowIndex);
      }
      rows.removeAll(removedRows);
      fireTableRowsDeleted(start, end);
    }
    return removedRows;
  }

  /**
   * Remove todos os objetos-linha.
   * 
   * @return coleo de objetos removidos.
   */
  public Collection<T> removeAll() {
    Collection<T> allRows = getRows();
    clear();
    return allRows;
  }

  /**
   * Modifica os dados de um objeto-linha da lista.
   * 
   * @param row objeto-linha a ser modificado.
   * 
   * @return true caso o objeto tenha sido modificado com sucesso, false caso
   *         este no tenha sido encontrado.
   * 
   * @throws IllegalArgumentException se a linha for null
   */
  public boolean modify(T row) {
    if (row == null) {
      throw new IllegalArgumentException("row == null");
    }
    int index = rows.indexOf(row);
    if (index == -1) {
      return false;
    }
    rows.set(index, row);
    fireTableRowsUpdated(index, index);
    return true;
  }

  /**
   * Obtm a lista de objetos-linha atualizada.
   * 
   * @return lista de objetos-linha.
   */
  public List<T> getRows() {
    return rows;
  }

  /**
   * Obtm um objeto-linha especfico.
   * 
   * @param rowIndex ndice do objeto-linha
   * 
   * @return o objeto-linha solicitado, ou null
   * 
   * @throws IndexOutOfBoundsException se o ndice for invlido
   */
  public T getRow(int rowIndex) {
    return rows.get(rowIndex);
  }

  /**
   * Troca duas linhas no modelo.
   * 
   * @param index1 - ndice da primeira linha
   * @param index2 - ndice da segunda linha
   */
  public void swapRows(int index1, int index2) {
    Collections.swap(getRows(), index1, index2);
    fireTableDataChanged();
  }

  /* Aqui terminam os mtodos para alterar a lista de objetos-linha. */
}

==> javautils-gui-5.0.3-sources.jar: tecgraf/javautils/gui/table/ColumnsObjectTableModel.java <==
package tecgraf.javautils.gui.table;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import javax.swing.event.TableModelEvent;
import javax.swing.table.AbstractTableModel;

/**
 * <p>
 * Modelo de tabela de objetos de fcil construo.<br>
 * Sua arquitetura permite reutilizar o cdigo das colunas e exime o
 * desenvolvedor de trabalhar com os ndices da tabela, seja de linha ou de
 * coluna.
 * </p>
 * <p>Para criar um modelo de tabela voc precisa de duas coisas:</p>
 * <ul>
 * <li>os modelos das colunas que podem ser criados implementando
 * {@link IColumn} ou estendendo {@link AbstractColumn}.</li>
 * <li>uma lista com os objetos que devero aparecer na tabela.</li>
 * </ul>
 * <p>
 * Obs. esse modelo no faz uso dos mtodos
 * {@link IColumn#createTableCellEditor()} e
 * {@link IColumn#createTableCellRenderer()}.
 * </p>
 * <p>
 * Antes de utilizar esse modelo d uma olhada na classe
 * {@link ObjectTableBuilder} que facilita a construo de tabelas cujas
 * informaes mostradas esto contidas em um objeto.
 * </p>
 * 
 * @param <T> Tipo dos objetos que sero representados por linhas da tabela.
 * 
 * @see ColumnsObjectTableModelSample 
 * 
 * @author Tecgraf
 */
public class ColumnsObjectTableModel<T> extends AbstractTableModel {
  /** Lista de objetos representando linhas da tabela */
  private List<T> rows;
  /** Modelo das colunas da tabela na ordem em que devem aparecer. */
  private IColumn<T>[] columns;

  /**
   * Cria o modelo.
   * 
   * @param rows lista de objetos-linha.
   * @param columns Modelo das colunas da tabela na ordem em que devem aparecer.
   * 
   * @throws IllegalArgumentException se a lista de linhas ou o provedor forem
   *         iguais a null, ou se a lista de nomes das colunas retornada pelo
   *         provedor for null
   */
  public ColumnsObjectTableModel(List<T> rows, IColumn<T>... columns) {
    if (rows == null) {
      throw new IllegalArgumentException("rows == null");
    }
    if (null == columns) {
      throw new IllegalArgumentException("columns == null");
    }
    if (columns.length == 0) {
      throw new IllegalArgumentException("columns.length == 0");
    }

    this.rows = rows;
    this.columns = columns;
  }

  /**
   * Obtm os modelos das colunas da tabela na ordem em que elas aparecem.<br>
   * Obs. mudanas nesse array e/ou nas colunas poderam danificar o
   * funcionamento da tabela.
   * 
   * @return os modelos das colunas da tabela na ordem em que elas aparecem.
   */
  public IColumn<T>[] getColumns() {
    return columns;
  }

  /* A partir daqui comea a implementao dos mtodos de AbstractTableModel. */

  /**
   * {@inheritDoc}
   */
  @Override
  public int getRowCount() {
    return rows.size();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public int getColumnCount() {
    return columns.length;
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Object getValueAt(int rowIndex, int columnIndex) {
    T row = rows.get(rowIndex);
    if (null == row) {
      throw new NullPointerException("rowIndex points to a null object.");
    }

    return columns[columnIndex].getValue(row);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void setValueAt(Object newValue, int rowIndex, int columnIndex) {
    T row = rows.get(rowIndex);
    if (null == row) {
      throw new NullPointerException("rowIndex points to a null object.");
    }

    columns[columnIndex].setValue(row, newValue);
    fireTableCellUpdated(rowIndex, columnIndex);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public boolean isCellEditable(int rowIndex, int columnIndex) {
    T row = rows.get(rowIndex);
    if (null == row) {
      throw new NullPointerException("rowIndex points to a null object.");
    }

    return columns[columnIndex].isEditable(row);
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public String getColumnName(int columnIndex) {
    return columns[columnIndex].getColumnName();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Class<?> getColumnClass(int columnIndex) {
    return columns[columnIndex].getColumnClass();
  }

  /* Aqui termina a implementao dos mtodos de AbstractTableModel. */

  /* A partir daqui comeam os mtodos para alterar a lista de objetos-linha. */

  /**
   * Remove todos os elementos de objetos.
   */
  public void clear() {
    int size = rows.size();
    this.rows = new ArrayList<T>();
    if (size > 0) {
      fireTableRowsDeleted(0, size - 1);
    }
  }

  /**
   * Substitui a lista de objetos-linha.
   * 
   * @param newRows nova lista de objetos-linha.
   * 
   * @throws IllegalArgumentException se a lista de novas linhas for null
   */
  public void setRows(List<T> newRows) {
    if (newRows == null) {
      throw new IllegalArgumentException("newRows == null");
    }
    this.rows = newRows;
    fireTableChanged(new TableModelEvent(this));
  }

  /**
   * Adiciona um objeto-linha  lista. Se j existir um objeto com o mesmo
   * identificador, este  sobrescrito (no permite duplicatas).
   * 
   * @param row objeto-linha a ser adicionado.
   * 
   * @throws IllegalArgumentException se <code>row</code> for nulo.
   */
  public void add(T row) {
    add(row, true);
  }

  /**
   * Adiciona uma nova linha  lista, com a possibilidade de notificar ou no a
   * tabela.
   * 
   * @param row nova linha
   * @param notify <code>true</code> se a tabela deve ser notificada da insero
   */
  private void add(T row, boolean notify) {
    if (row == null) {
      throw new IllegalArgumentException("row == null");
    }
    int index = rows.indexOf(row);
    if (index != -1) {
      rows.set(index, row);
      if (notify) {
        fireTableRowsUpdated(index, index);
      }
    }
    else {
      rows.add(row);
      if (notify) {
        index = rows.size() - 1;
        fireTableRowsInserted(index, index);
      }
    }
  }

  /**
   * Adiciona uma coleo de objetos-linha  lista. Se j existir um objeto com
   * o mesmo identificador, este  sobrescrito (no permite duplicatas).
   * 
   * @param rows coleo de objetos-linha a serem adicionados.
   * 
   * @throws IllegalArgumentException se <code>rows</code> for nulo.
   */
  public void addAll(Collection<T> rows) {
    if (rows == null) {
      throw new IllegalArgumentException("rows == null");
    }
    for (T row : rows) {
      /*
       * inserimos a linha sem notificar a tabela, pois faremos uma nica
       * notificao aps o loop
       */
      add(row, false);
    }
    fireTableChanged(new TableModelEvent(this));
  }

  /**
   * Remove um objeto-linha da lista.
   * 
   * @param row objeto-linha a ser removido.
   * 
   * @return true se o objeto-linha for encontrado e removido com sucesso, false
   *         caso contrrio.
   * 
   * @throws IllegalArgumentException se <code>row</code> for nulo.
   */
  public boolean remove(T row) {
    if (row == null) {
      throw new IllegalArgumentException("row == null");
    }
    int index = rows.indexOf(row);
    if (index == -1) {
      return false;
    }
    rows.remove(index);
    fireTableRowsDeleted(index, index);
    return true;
  }

  /**
   * Remove um objeto-linha da lista pelo seu ndice do modelo.
   * 
   * @param rowIndex ndice do objeto-linha a ser removido.
   * 
   * @return o objeto-linha removido.
   * 
   * @throws IndexOutOfBoundsException se o ndice for negativo ou se for maior
   *         do que o tamanho da lista de objeto-linha.
   */
  public T remove(int rowIndex) {
    T row = rows.remove(rowIndex);
    fireTableRowsDeleted(rowIndex, rowIndex);
    return row;
  }

  /**
   * Remove os objetos-linha especificados pelos seus ndices.
   * 
   * @see AbstractTableModel#fireTableRowsDeleted(int, int)
   * @param rowIndexes ndices dos objetos-linha a serem removidos.
   * 
   * @return coleo de objetos removidos.
   * 
   * @throws IllegalArgumentException se <code>rowIndexes</code> for nulo.
   * @throws IndexOutOfBoundsException se algum ndice for negativo ou se for
   *         maior do que o tamanho da lista de objeto-linha.
   * 
   */
  public Collection<T> removeAll(int[] rowIndexes) {
    if (rowIndexes == null) {
      throw new IllegalArgumentException("rowIndexes == null");
    }
    List<T> removedRows = new ArrayList<T>();
    if (rowIndexes.length > 0) {
      /*
       * devemos sinalizar mudanas apenas no intervalo que compreende as linhas
       * removidas (inclusive). Para isso, precisamos achar o menor e o maior
       * ndices dentre os reportados.
       */
      int start = Integer.MAX_VALUE, end = 0;
      for (int rowIndex : rowIndexes) {
        removedRows.add(rows.get(rowIndex));
        start = Math.min(start, rowIndex);
        end = Math.max(end, rowIndex);
      }
      rows.removeAll(removedRows);
      fireTableRowsDeleted(start, end);
    }
    return removedRows;
  }

  /**
   * Remove todos os objetos-linha.
   * 
   * @return coleo de objetos removidos.
   */
  public Collection<T> removeAll() {
    Collection<T> allRows = getRows();
    clear();
    return allRows;
  }

  /**
   * Modifica os dados de um objeto-linha da lista.
   * 
   * @param row objeto-linha a ser modificado.
   * 
   * @return true caso o objeto tenha sido modificado com sucesso, false caso
   *         este no tenha sido encontrado.
   * 
   * @throws IllegalArgumentException se a linha for null
   */
  public boolean modify(T row) {
    if (row == null) {
      throw new IllegalArgumentException("row == null");
    }
    int index = rows.indexOf(row);
    if (index == -1) {
      return false;
    }
    rows.set(index, row);
    fireTableRowsUpdated(index, index);
    return true;
  }

  /**
   * Obtm a lista de objetos-linha atualizada.
   * 
   * @return lista de objetos-linha.
   */
  public List<T> getRows() {
    return rows;
  }

  /**
   * Obtm um objeto-linha especfico.
   * 
   * @param rowIndex ndice do objeto-linha
   * 
   * @return o objeto-linha solicitado, ou null
   * 
   * @throws IndexOutOfBoundsException se o ndice for invlido
   */
  public T getRow(int rowIndex) {
    return rows.get(rowIndex);
  }

  /**
   * Troca duas linhas no modelo.
   * 
   * @param index1 - ndice da primeira linha
   * @param index2 - ndice da segunda linha
   */
  public void swapRows(int index1, int index2) {
    Collections.swap(getRows(), index1, index2);
    fireTableDataChanged();
  }

  /* Aqui terminam os mtodos para alterar a lista de objetos-linha. */
}
